/* $Id: lp.h,v 1.5 92/11/17 20:13:52 wolf Exp $
 *
 *  Copyright (C) 1991,1992  Kimmo Suominen, Christophe Wolfhugel
 *
 *  Please read the files COPYRIGHT and AUTHORS for the extended
 *  copyrights refering to this file.
 *
 * $Log:	lp.h,v $
 * Revision 1.5  92/11/17  20:13:52  wolf
 * Corrected horrible bug for IsXXX(). Proto moved from
 * char * to int as in lp.c
 * 
 * Revision 1.4  92/06/12  07:11:28  listserv
 * Initial 2.00 release
 * 
 * 
 * External definitions from lp.c
 */

extern int IsEditor(char *);
extern int IsOwner(char *);
extern int IsUser(char *);

extern void RewindUserList();
extern void RewindCommentList();
extern void RewindEditorList();
extern void RewindOwnerList();

extern int AddUser(char *);
extern int DelUser(char *);

extern void CloseUserList();
extern int ReadUserList(char *);
extern void WriteUserList();

extern char *GetUser(char *);
extern char *GetComment(char *);
extern char *GetEditor(char *);
extern char *GetOwner(char *);

extern char *GetErrorsTo();
extern char *GetReplyTo();
extern char *GetReview();
extern char *GetSend();
extern char *GetSubscription();
